Player = Class{}

local PLAYER_INIT_X = 74
local PLAYER_INIT_Y = 616

local SCAN = {
	X = 'x',
	Y = 'y'
}

PLAYER_SPEED = 600


function Player:init(x, y, sprite)
	self.x = x
	self.y = y

	self.dx = PLAYER_SPEED
	self.t = 0
	self.ti = 0
	self.tf = 0
	self.trajectory = {}

	self.grounded = false
	self.jumped = false
	self.falling = false

	self.rotation = 0


	self.sprite = sprite

	self.width = sprite:getWidth()
	self.height = sprite:getHeight()

	self.screenOffset = self.x

	self:updateBounds()


end

function Player:update(dt)

	self.x = round(self.x + self.dx * dt)
	
	if self.x < -world.translation + self.screenOffset then
		self.x = round(approach(self.x, PLAYER_SPEED * dt / 2, -world.translation + self.screenOffset))
	end

	self:updateBounds()


	for k, ground in pairs(world.ground) do
		if collidesAABB(ground.bounds, self.AABBBounds) then
			self:hitGround(ground, SCAN.X)
			break
		end
	end

	self:updateBounds()

	if love.keyboard.wasPressed('up') and not self.jumped then
		self.trajectory = self:calculateTrajectory(self.y, self.y - 150, self.x, self.x + 25)
		self.t = self.x
		self.ti = self.t
		self.tf = self:findGround(self.trajectory, world.ground)
		self.jumped = true
		self.falling = true
	end

	if not self:checkGround(world.ground) and not self.falling then
		self.trajectory = self:calculateTrajectory(self.y + 150, self.y, self.x + 25, self.x)
		self.t = self.x
		self.ti = self.t
		self.tf = self:findGround(self.trajectory, world.ground)
		self.falling = true
	end

	if tableLength(self.trajectory) ~= 0 then
		self.t = self.t + dt * 100
		self.rotation = math.pi * 2 * 1.125 * ((self.t - self.ti) / (self.tf - self.ti))
		self.y = round(self:followTrajectory(self.trajectory, self.t))
	end

	self:updateBounds()

	for k, ground in pairs(world.ground) do
		if collidesAABB(ground.bounds, self.AABBBounds) then
			self:hitGround(ground, SCAN.Y)
			break
		end
	end

	self:updateBounds()

	for k, spike in pairs(world.spikes) do
		if collidesCircleTriangle(self.circleBounds, spike.bounds) then
			self:hitSpike(spike)
		end
	end
end

function Player:hitGround(ground, scanDir)
	self.rotation = 0

	if scanDir == SCAN.X then
		local xPenetration = 0
		if ground.x > self.x then
			xPenetration = self.AABBBounds.right - ground.bounds.left
		else
			xPenetration = self.AABBBounds.left - ground.bounds.right
		end
		self.x = round(self.x - xPenetration) - 1
	end

	if scanDir == SCAN.Y then
		self.t = self.tf
		self.trajectory = {}
		self.jumped = false
		self.falling = false
		local yPenetration = 0
		if ground.y > self.y then
			yPenetration = self.AABBBounds.bottom - ground.bounds.top
			self.y = round(self.y - yPenetration) - 1
		else 
			yPenetration = ground.bounds.bottom - self.AABBBounds.top
			self.y = round(self.y + yPenetration) + 1
		end

	end
end

function Player:checkGround(ground)
	for k, grnd in pairs(ground) do
		local checkPoint = {
			x = self.x + self.width / 2, 
			y = self.AABBBounds.bottom + 1
		}
		if collidesAABBPoint(grnd.bounds, checkPoint) then
			return true
		end
	end

	return false
end

function Player:calculateTrajectory(y, k, x, h)
	local a = (y - k) / ((x - h) ^ 2)
	return {
		a = a,
		h = h,
		k = k
	}
end

function Player:followTrajectory(traj, t)
	return round(traj.a * ((t - traj.h) ^ 2) + traj.k)
end

function Player:findGround(traj, ground)
	local t = self.x
	local point = {x = t, y = self:followTrajectory(traj, t)}
	local lastY = point.y
	local dy = 0
	while point.y < WINDOW_HEIGHT and point.y > 0 do
		for k, grnd in pairs(ground) do
			if collidesAABBPoint(grnd.bounds, point) then
				if dy > 0 then
					return point.x
				end
			end
		end
		lastY = point.y
		t = t + 1
		point = {x = t, y = self:followTrajectory(traj, t)}
		dy = point.y - lastY
		debug = dy
	end

	return t
end



function Player:hitSpike(spike)
	self.dx = 0
	self.trajectory = {}
end


function Player:updateBounds()
	self.center = {
		x = self.x + self.width / 2,
		y = self.y + self.height / 2
	}

	self.circleBounds = {
		center = {x = self.center.x, y = self.center.y},
		radius = self.width / 2 - 2
	}
	self.AABBBounds = {
		top = self.y,
		right = self.x + self.width - 1,
		bottom = self.y + self.height - 1,
		left = self.x
	}
end

function Player:render(skew)
	--local shear = self.dx ^ 2 * (0.4 / PLAYER_SPEED ^ 2) * sign(self.dx)
	love.graphics.setColor(255, 255, 255, 255)
	if skew then
		love.graphics.draw(self.sprite, self.x+self.width/2, self.y+self.height/2, self.rotation, 1, -1, self.width/2, self.height/2, skew, 0)
	else
		love.graphics.draw(self.sprite, self.x+self.width/2, self.y+self.height/2, self.rotation, 1, 1, self.width/2, self.height/2)
	end
	love.graphics.setColor(0, 255, 0, 255)
	--love.graphics.circle('line', self.circleBounds.center.x, self.circleBounds.center.y, self.circleBounds.radius)
	--love.graphics.rectangle('line', self.AABBBounds.left, self.AABBBounds.top, self.width, self.height)

end