-- returns the absolute value of a number
function abs(value)
    if (value < 0) then
        return -value
    end
    return value
end

function approach(a, f, target)
    if sign(a-target) == 1 then
        return math.max(a-abs(f), target)
    else
        return math.min(a+abs(f), target)
    end
end

-- clamps value between the range of min and max, inclusive
function clamp(value, min, max)
    if (value < min) then
        return min
    end
    if (value > max) then
        return max
    end
    return value
end

-- linearly interpolates value from itself to target by a factor
-- clamps value between value and target
function lerp(value, target, factor)
    if (value <= target) then
        return clamp(value + (target - value) * factor, value, target)
    else
        return clamp(value + (target - value) * factor, target, value)
    end
end

function inverseLerp(value, target, factor)
    if (value <= target) then
        return clamp(target - (target - value) * factor, value, target)
    else
        return clamp(target - (target - value) * factor, target, value)
    end
end

function mapQuadratic(value, min1, max1, min2, max2)
    slope = (max2 - min2) / (max1 - min1)
    return min2 + slope * (value - min1)
end

function round(value)
    if value * 10 % 10 >= 5 then
        return math.ceil(value)
    else
        return math.floor(value)
    end
end

-- returns the sign of a number, -1 for negative, 1 for positive, and 0 for zero
function sign(value)
    if value == 0 then
        return 0
    end
    if value > 0 then
        return 1
    end
    if (value < 0) then
        return -1
    end
end

function signbi(value)
    if value >= 0 then
        return 1
    end
    if value < 0 then
        return -1
    end
end

-- implementation of C-like ternary operator
function ternary(a, b, c)
    if (a) then
        return b
    else
        return c
    end
end


function tableLength(table)
    local count = 0
    for k in pairs(table) do
        count = count + 1
    end
    return count
end


--[[ VECTOR ]]

function distance(v1, v2)
    return math.sqrt((v1.x - v2.x) ^ 2 + (v1.y - v2.y) ^ 2)
end

function between(point, range)
    local xBetween = false
    local yBetween = false

    if range.neg.x < point.x and point.x < range.pos.x then
        xBetween = true
    end 

    if range.neg.y < point.y and point.y < range.pos.y then
        yBetween = true
    end

    return xBetween, yBetween
end

function distancePointLine(point, line)
    local slope = (line.pos.y - line.neg.y) / (line.pos.x - line.neg.x)
    local perpSlope = -1 / slope
    local xIntersect = (slope * line.pos.x - perpSlope * point.x + point.y - line.pos.y) / (slope - perpSlope)
    if line.neg.x > line.pos.x then
        if line.pos.x > xIntersect or xIntersect > line.neg.x then
            return -1
        end
    else
        if line.neg.x > xIntersect or xIntersect > line.pos.x then
            return -1
        end
    end
    local yIntersect = perpSlope * (xIntersect - point.x) + point.y
    return distance(point, {x = xIntersect, y = yIntersect})
end