function collidesAABB(AABB1, AABB2)
    if AABB1.left > AABB2.right or AABB2.left > AABB1.right then
        return false
    end

    if AABB1.top > AABB2.bottom or AABB2.top > AABB1.bottom then
        return false
    end 

    return true
end

function collidesAABBPoint(AABB, point)
	return point.x <= AABB.right and point.x >= AABB.left and
		point.y <= AABB.bottom and point.y >= AABB.top
end

function collidesAABBCircle(AABB, circle)

	if collidesAABBPoint(AABB, circle.center) then
		return true
	end

	local corners = {
		topRight = {x = AABB.right, y = AABB.top},
		bottomRight = {x = AABB.right, y = AABB.bottom},
		bottomLeft = {x = AABB.left, y = AABB.bottom},
		topLeft = {x = AABB.left, y = AABB.top}
	}

	for k, corner in pairs(corners) do
		if distance(corner, circle.center) < circle.radius then
			return true
		end
	end

	local sides = {
		top = {neg = corners.topLeft, pos = corners.topRight},
		right = {neg = corners.topRight, pos = corners.bottomRight},
		bottom = {neg = corners.bottomLeft, pos = corners.bottomRight},
		left = {neg = corners.topLeft, pos = corners.bottomLeft}
	}

	for k, side in pairs(sides) do
		local xBetween, yBetween = between(circle.center, side)
		if xBetween then
			if 	side.neg.y < circle.center.y + circle.radius and circle.center.y + circle.radius < side.pos.y or
				side.neg.y < circle.center.y - circle.radius and circle.center.y - circle.radius < side.pos.y then
				return true
			end
		end
		if yBetween then
			if 	side.neg.x < circle.center.x + circle.radius and circle.center.x + circle.radius < side.pos.x or
				side.neg.x < circle.center.x - circle.radius and circle.center.x - circle.radius < side.pos.x then
				return true
			end
		end
	end

end

function collidesAABBTriangle(AABB, tri)
	local function height(x)
		if x < tri.top.x then
			local slope = 	(tri.top.y - tri.left.y) /
							(tri.top.x - tri.left.x)
			return -(x - tri.left.x) * slope
		elseif x > tri.top.x then
			local slope =	(tri.top.y - tri.right.y) /
							(tri.right.x - tri.top.x)
			return -(tri.right.x - x) * slope
		else
			return -tri.top.y
		end
	end

	if AABB.right - tri.left.x < tri.top.x - tri.left.x and AABB.right > tri.left.x then
		if tri.right.y - AABB.bottom < height(AABB.right) and tri.right.y - AABB.bottom >= 0 then
			return true
		end
	end

	if tri.right.x - AABB.left < tri.right.x - tri.top.x and AABB.left < tri.right.x then
		if tri.right.y - AABB.bottom < height(AABB.left) and tri.right.y - AABB.bottom >= 0 then
			return true
		end
	end

	if AABB.right > tri.top.x and AABB.left < tri.top.x then
		if tri.right.y - AABB.bottom < tri.right.y - tri.top.y and tri.right.y - AABB.bottom >= 0 then
			return true
		end
	end

	return false
end

function collidesPointTriangle(point, tri)
	local function height(x)
		if x < tri.top.x then
			local slope = 	(tri.top.y - tri.left.y) /
							(tri.top.x - tri.left.x)
			return -(x - tri.left.x) * slope
		elseif x > tri.top.x then
			local slope =	(tri.top.y - tri.right.y) /
							(tri.right.x - tri.top.x)
			return -(tri.right.x - x) * slope
		else
			return -tri.top.y
		end
	end

	if point.x > tri.left.x and point.x < tri.right.x then
		if tri.right.y - point.y < height(point.x) and point.y < tri.right.y then
			return true
		else 
			return false
		end
	else 
		return false
	end
end

function collidesCircleTriangle(circle, tri)
	if collidesPointTriangle(circle.center, tri) then
		return true
	end

	for k, vertex in pairs(tri) do
		if distance(circle.center, vertex) < circle.radius then
			return true
		end
	end

	local sides = {
		[0] = {neg = tri.top, pos = tri.right},
		[1] = {neg = tri.left, pos = tri.top}
	}

	for k, side in pairs(sides) do
		local dist = distancePointLine(circle.center, sides[k])
		if dist < circle.radius and dist > 0 then
			return true
		end
	end

	return false
end