LevelEditorState = Class{__includes = BaseState}

local TILE_SIZE = 32

function LevelEditorState:init()
	levelEditor = self
	self.divide = BRUSH_ROWS * BRUSH_SIZE

	self.brushPalette = {}
	self.brushMap = {}
	local paletteCount = 1
	for k, sprite in pairs(gSprites) do
		local brushX = (paletteCount - 1) % BRUSH_ROWS * BRUSH_SIZE
		local brushY = math.floor((paletteCount - 1) / BRUSH_ROWS) * BRUSH_SIZE
		self.brushPalette[paletteCount] = Brush(brushX, brushY, sprite)
		self.brushMap[sprite] = paletteCount
		paletteCount = paletteCount + 1
	end

	self.scroll = {
		x = self.divide,
		y = VIRTUAL_HEIGHT
	}

	self.lastMousePosition = {
		x = love.mouse.getX(),
		y = love.mouse.getY()
	}

	self.currentBrush = nil

	self.objects = {}

	self.backButton = {
		top = VIRTUAL_HEIGHT - 100,
		right = 108,
		bottom = VIRTUAL_HEIGHT - 50,
		left = 20
	}
end

-- function love.wheelmoved(x, y)
-- 	TILE_SIZE = TILE_SIZE + y
-- 	for k, object in pairs(levelEditor.objects) do
-- 		object.x = levelEditor.scroll.x + (object.x - levelEditor.scroll.x) / (TILE_SIZE - y) * TILE_SIZE
-- 		object.y = levelEditor.scroll.y + (object.y - levelEditor.scroll.y) / (TILE_SIZE - y) * TILE_SIZE
-- 		object.bounds = {	
-- 			top = object.y, 
-- 			right = object.x + object.sprite:getWidth() * TILE_SIZE / 32 - 1, 
-- 			bottom = object.y + object.sprite:getHeight() * TILE_SIZE / 32 - 1, 
-- 			left = object.x
-- 		}
-- 	end
-- 	debug = y
-- end

function LevelEditorState:update(dt)
	if love.keyboard.wasPressed('enter') or love.keyboard.wasPressed('return') then
		self:saveLevel()
	end

	if love.keyboard.wasPressed('c') then
		self.objects = {}
	end

	if love.mouse.wheelInput ~= 0 then
		TILE_SIZE = TILE_SIZE + love.mouse.wheelInput
		for k, object in pairs(self.objects) do
			object.x = self.scroll.x + (object.x - self.scroll.x) / (TILE_SIZE - love.mouse.wheelInput) * TILE_SIZE
			object.y = self.scroll.y + (object.y - self.scroll.y) / (TILE_SIZE - love.mouse.wheelInput) * TILE_SIZE
			object.bounds = {	
				top = object.y, 
				right = object.x + object.sprite:getWidth() * TILE_SIZE / 32 - 1, 
				bottom = object.y + object.sprite:getHeight() * TILE_SIZE / 32 - 1, 
				left = object.x
			}
		end
		debug = "SDF"
	end

	local mousePos = {x = love.mouse.getX(), y = love.mouse.getY()}

	if love.mouse.wasPressed(1) and collidesAABBPoint(self.backButton, mousePos) then
		gStateMachine:change('start')
	end

	if love.mouse.wasPressed(1) and mousePos.x < self.divide then

		self.currentBrush = nil
		for k, brush in pairs(self.brushPalette) do
			if collidesAABBPoint(brush.bounds, mousePos) then
				brush:toggle()
				if brush.selected then
					self.currentBrush = brush.sprite
				end
			else
				brush:deselect()
			end
		end
	end


	if love.mouse.wasPressed(2) and self.currentBrush == nil then
		for k, object in pairs(self.objects) do
			if collidesAABBPoint(object.bounds, mousePos) then
				table.remove(self.objects, k)
			end
		end
	end

	if love.mouse.wasPressed(2) and self.currentBrush ~= nil then
		for k, brush in pairs(self.brushPalette) do
			if brush.selected then
				brush:deselect()
				self.currentBrush = nil
				break
			end
		end
	end

	if love.mouse.isDown(1) and mousePos.x > self.divide and self.currentBrush ~= nil then
		local obj = {
			x = mousePos.x - (mousePos.x - self.scroll.x) % TILE_SIZE,
			y = mousePos.y - (mousePos.y - self.scroll.y) % TILE_SIZE,
			sprite = self.currentBrush
		}

		obj.bounds = {
			top = obj.y, 
			right = obj.x + obj.sprite:getWidth() * TILE_SIZE / 32 - 1, 
			bottom = obj.y + obj.sprite:getHeight() * TILE_SIZE / 32 - 1, 
			left = obj.x
		}

		for k, object in pairs(self.objects) do
			if collidesAABB(object.bounds, obj.bounds) then
				goto continue
			end
		end

		table.insert(self.objects, obj)
		::continue::
		debug = obj.x
	end

	if love.mouse.isDown(1) and mousePos.x > self.divide and self.currentBrush == nil  or love.mouse.isDown(3) then
		self.scroll = {
			x = self.scroll.x + mousePos.x - self.lastMousePos.x,
			y = self.scroll.y + mousePos.y - self.lastMousePos.y
		}
		for k, object in pairs(self.objects) do
			object.x = object.x + mousePos.x - self.lastMousePos.x
			object.y = object.y + mousePos.y - self.lastMousePos.y
			debug = string.format('%f %f', object.x - self.scroll.x, object.y - self.scroll.y + VIRTUAL_HEIGHT)
		end
	end

	self.lastMousePos = {x = mousePos.x, y = mousePos.y}

	for k, object in pairs(self.objects) do

		object.bounds = {	
			top = object.y, 
			right = object.x + object.sprite:getWidth() * TILE_SIZE / 32 - 1, 
			bottom = object.y + object.sprite:getHeight() * TILE_SIZE / 32 - 1, 
			left = object.x
		}
	end
end

function LevelEditorState:render()

	-- right half of level editor
	local startX = self.divide
	local width = VIRTUAL_WIDTH - startX
	love.graphics.setColor(131, 192, 240, 255)
	love.graphics.rectangle('fill', startX, 0, width, VIRTUAL_HEIGHT)

	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.setLineStyle('rough')
	for i = 0, VIRTUAL_WIDTH, TILE_SIZE do
		local lineX = i + self.scroll.x % TILE_SIZE
		if lineX - self.scroll.x == 0 then
			love.graphics.setLineWidth(3)
		else
			love.graphics.setLineWidth(1)
		end
		if (lineX > self.divide) then
			love.graphics.line(lineX, 0, lineX, VIRTUAL_HEIGHT)
		end
	end
	for i = 0, VIRTUAL_HEIGHT, TILE_SIZE do
		local lineY = i + self.scroll.y % TILE_SIZE
		if lineY - self.scroll.y == 0 then
			love.graphics.setLineWidth(3)
		else
			love.graphics.setLineWidth(1)
		end
		love.graphics.line(startX, lineY, VIRTUAL_WIDTH, lineY)
	end

	for k, object in pairs(self.objects) do
		love.graphics.draw(object.sprite, object.x, object.y, 0, TILE_SIZE / 32, TILE_SIZE / 32)
	end

	if love.mouse.getX() > self.divide and self.currentBrush ~= nil then
		love.graphics.draw(self.currentBrush, love.mouse.getX(), love.mouse.getY(), 0, TILE_SIZE / 32, TILE_SIZE / 32)
	end


	-- Left half of level editor
	love.graphics.setColor(172, 157, 129, 255)
	love.graphics.rectangle('fill', 0, 0, self.divide, VIRTUAL_HEIGHT)

	for k, brush in pairs(self.brushPalette) do
		brush:render()
	end

	love.graphics.setColor(255, 255, 255, 255)
	love.graphics.rectangle('fill', self.backButton.left, self.backButton.top, self.backButton.right - self.backButton.left, self.backButton.bottom - self.backButton.top)

	love.graphics.print(debug, 10, 200)
end

function LevelEditorState:saveLevel()
	love.filesystem.setIdentity("Speed is King")

	local fileName = ""
	for i = 0, 100 do
		fileName = "level" .. i .. ".txt"
		if not love.filesystem.exists(fileName) then
			break
		end
	end
	
	if fileName == "" then
		return
	end

	love.filesystem.write(fileName, "")

	for k, object in pairs(self.objects) do
		local realX = (object.x - self.scroll.x) / TILE_SIZE * 32
		local realY = (object.y - self.scroll.y) / TILE_SIZE * 32 + VIRTUAL_HEIGHT
		local objectData = self.brushMap[object.sprite] .. "\n" .. realX .. "\n" .. realY .. "\n"  .. "\n"
		love.filesystem.append(fileName, objectData)
	end
end