PlayState = Class{__includes = BaseState}

local PLAYER = 1
local SMALL_SPIKE = 2
local BIG_SPIKE = 3
local GROUND = 4

function PlayState:init()
	world = self
	self.translation = 0
end

function PlayState:enter(level)
	PlayState:loadLevel(level)
	self.level = level
end

function PlayState:loadLevel(level)
	love.filesystem.setIdentity('Speed is King')

	self.player = nil
	self.spikes = {}
	self.ground = {}

	local counter = 0
	local objectData = {}
	local fileName = 'level' .. level .. '.txt'
	if love.filesystem.exists(fileName) then
		for line in love.filesystem.lines(fileName) do
			if counter == 3 then
				if objectData['object'] == PLAYER then
					self.player = Player(objectData['x'], objectData['y'], gSprites['player'])
				end
				if objectData['object'] == SMALL_SPIKE then
					table.insert(self.spikes, Spike(objectData['x'], objectData['y'], gSprites['spike32x32']))
				end
				if objectData['object'] == BIG_SPIKE then
					table.insert(self.spikes, Spike(objectData['x'], objectData['y'], gSprites['spike64x64']))
				end
				if objectData['object'] == GROUND then
					table.insert(self.ground, Ground(objectData['x'], objectData['y'], gSprites['ground']))
				end
			end

			if counter == 0 then
				objectData['object'] = tonumber(line)
			end
			if counter == 1 then
				objectData['x'] = tonumber(line)
			end
			if counter == 2 then
				objectData['y'] = tonumber(line)
			end
			
			counter = counter + 1
			if counter == 4 then
				counter = 0
			end
		end
	end
end

function PlayState:update(dt)
	self.translation = round(self.translation - PLAYER_SPEED * dt)
	self.player:update(dt)

	if love.keyboard.wasPressed('return') then
		gStateMachine:change('play', self.level)
	end

	if love.keyboard.wasPressed('w') then
		gStateMachine:change('play', self.level + 1)
	end
	if love.keyboard.wasPressed('s') then
		gStateMachine:change('play', self.level - 1)
	end

	if love.mouse.wasPressed(1) then
		gStateMachine:change('start')
	end
end

function PlayState:render()
	love.graphics.push()
	
		love.graphics.translate(self.translation, 0)

		love.graphics.clear(120, 120, 120)

		for k, spike in pairs(self.spikes) do
			spike:render()
		end

		for k, ground in pairs(self.ground) do
			ground:render()
		end
		self.player:render()

	love.graphics.pop()

	love.graphics.print(debug, 10, 500)
end